@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: curl.h,v 1.52 2001/02/07 09:49:06 bagder Exp $
+ * $Id: curl.h,v 1.69 2001/03/22 11:40:58 bagder Exp $
  *****************************************************************************/
 
 #include <stdio.h>
@@ -97,68 +97,57 @@ typedef int (*curl_passwd_callback)(void *clientp,
 
 typedef enum {
   CURLE_OK = 0,
-  CURLE_UNSUPPORTED_PROTOCOL,
-  CURLE_FAILED_INIT,
-  CURLE_URL_MALFORMAT,
-  CURLE_URL_MALFORMAT_USER,
-  CURLE_COULDNT_RESOLVE_PROXY,
-  CURLE_COULDNT_RESOLVE_HOST,
-  CURLE_COULDNT_CONNECT,
-  CURLE_FTP_WEIRD_SERVER_REPLY,
-  CURLE_FTP_ACCESS_DENIED,
-  CURLE_FTP_USER_PASSWORD_INCORRECT,
-  CURLE_FTP_WEIRD_PASS_REPLY,
-  CURLE_FTP_WEIRD_USER_REPLY,
-  CURLE_FTP_WEIRD_PASV_REPLY,
-  CURLE_FTP_WEIRD_227_FORMAT,
-  CURLE_FTP_CANT_GET_HOST,
-  CURLE_FTP_CANT_RECONNECT,
-  CURLE_FTP_COULDNT_SET_BINARY,
-  CURLE_PARTIAL_FILE,
-  CURLE_FTP_COULDNT_RETR_FILE,
-  CURLE_FTP_WRITE_ERROR,
-  CURLE_FTP_QUOTE_ERROR,
-  CURLE_HTTP_NOT_FOUND,
-  CURLE_WRITE_ERROR,
+  CURLE_UNSUPPORTED_PROTOCOL,    /* 1 */
+  CURLE_FAILED_INIT,             /* 2 */
+  CURLE_URL_MALFORMAT,           /* 3 */
+  CURLE_URL_MALFORMAT_USER,      /* 4 */
+  CURLE_COULDNT_RESOLVE_PROXY,   /* 5 */
+  CURLE_COULDNT_RESOLVE_HOST,    /* 6 */
+  CURLE_COULDNT_CONNECT,         /* 7 */
+  CURLE_FTP_WEIRD_SERVER_REPLY,  /* 8 */
+  CURLE_FTP_ACCESS_DENIED,       /* 9 */
+  CURLE_FTP_USER_PASSWORD_INCORRECT, /* 10 */
+  CURLE_FTP_WEIRD_PASS_REPLY,    /* 11 */
+  CURLE_FTP_WEIRD_USER_REPLY,    /* 12 */
+  CURLE_FTP_WEIRD_PASV_REPLY,    /* 13 */
+  CURLE_FTP_WEIRD_227_FORMAT,    /* 14 */
+  CURLE_FTP_CANT_GET_HOST,       /* 15 */
+  CURLE_FTP_CANT_RECONNECT,      /* 16 */
+  CURLE_FTP_COULDNT_SET_BINARY,  /* 17 */
+  CURLE_PARTIAL_FILE,            /* 18 */
+  CURLE_FTP_COULDNT_RETR_FILE,   /* 19 */
+  CURLE_FTP_WRITE_ERROR,         /* 20 */
+  CURLE_FTP_QUOTE_ERROR,         /* 21 */
+  CURLE_HTTP_NOT_FOUND,          /* 22 */
+  CURLE_WRITE_ERROR,             /* 23 */
+  CURLE_MALFORMAT_USER,          /* 24 - user name is illegally specified */
+  CURLE_FTP_COULDNT_STOR_FILE,   /* 25 - failed FTP upload */
+  CURLE_READ_ERROR,              /* 26 - could open/read from file */
+  CURLE_OUT_OF_MEMORY,           /* 27 */
+  CURLE_OPERATION_TIMEOUTED,     /* 28 - the timeout time was reached */
+  CURLE_FTP_COULDNT_SET_ASCII,   /* 29 - TYPE A failed */
+  CURLE_FTP_PORT_FAILED,         /* 30 - FTP PORT operation failed */
+  CURLE_FTP_COULDNT_USE_REST,    /* 31 - the REST command failed */
+  CURLE_FTP_COULDNT_GET_SIZE,    /* 32 - the SIZE command failed */
+  CURLE_HTTP_RANGE_ERROR,        /* 33 - RANGE "command" didn't work */
+  CURLE_HTTP_POST_ERROR,         /* 34 */
+  CURLE_SSL_CONNECT_ERROR,       /* 35 - wrong when connecting with SSL */
+  CURLE_FTP_BAD_DOWNLOAD_RESUME, /* 36 - couldn't resume download */
+  CURLE_FILE_COULDNT_READ_FILE,  /* 37 */
+  CURLE_LDAP_CANNOT_BIND,        /* 38 */
+  CURLE_LDAP_SEARCH_FAILED,      /* 39 */
+  CURLE_LIBRARY_NOT_FOUND,       /* 40 */
+  CURLE_FUNCTION_NOT_FOUND,      /* 41 */
+  CURLE_ABORTED_BY_CALLBACK,     /* 42 */
+  CURLE_BAD_FUNCTION_ARGUMENT,   /* 43 */
+  CURLE_BAD_CALLING_ORDER,       /* 44 */
+  CURLE_HTTP_PORT_FAILED,        /* 45 - HTTP Interface operation failed */
+  CURLE_BAD_PASSWORD_ENTERED,    /* 46 - my_getpass() returns fail */
+  CURLE_TOO_MANY_REDIRECTS ,     /* 47 - catch endless re-direct loops */
+  CURLE_UNKNOWN_TELNET_OPTION,   /* 48 - User specified an unknown option */
+  CURLE_TELNET_OPTION_SYNTAX ,   /* 49 - Malformed telnet option */
 
-  CURLE_MALFORMAT_USER, /* the user name is illegally specified */
-  CURLE_FTP_COULDNT_STOR_FILE, /* failed FTP upload */
-  CURLE_READ_ERROR, /* could open/read from file */
-
-  CURLE_OUT_OF_MEMORY,
-  CURLE_OPERATION_TIMEOUTED, /* the timeout time was reached */
-  CURLE_FTP_COULDNT_SET_ASCII, /* TYPE A failed */
-
-  CURLE_FTP_PORT_FAILED, /* FTP PORT operation failed */
-
-  CURLE_FTP_COULDNT_USE_REST, /* the REST command failed */
-  CURLE_FTP_COULDNT_GET_SIZE, /* the SIZE command failed */
-
-  CURLE_HTTP_RANGE_ERROR, /* The RANGE "command" didn't seem to work */
-
-  CURLE_HTTP_POST_ERROR,
-
-  CURLE_SSL_CONNECT_ERROR, /* something was wrong when connecting with SSL */
-
-  CURLE_FTP_BAD_DOWNLOAD_RESUME, /* couldn't resume download */
-
-  CURLE_FILE_COULDNT_READ_FILE,
-
-  CURLE_LDAP_CANNOT_BIND,
-  CURLE_LDAP_SEARCH_FAILED,
-  CURLE_LIBRARY_NOT_FOUND,
-  CURLE_FUNCTION_NOT_FOUND,
-  
-  CURLE_ABORTED_BY_CALLBACK,
-  CURLE_BAD_FUNCTION_ARGUMENT,
-  CURLE_BAD_CALLING_ORDER,
-
-  CURLE_HTTP_PORT_FAILED, /* HTTP Interface operation failed */
-
-  CURLE_BAD_PASSWORD_ENTERED, /* when the my_getpass() returns fail */
-  CURLE_TOO_MANY_REDIRECTS , /* catch endless re-direct loops */
-
-  CURL_LAST
+  CURL_LAST /* never use! */
 } CURLcode;
 
 /* This is just to make older programs not break: */
@@ -406,6 +395,41 @@ typedef enum {
      document! Pass a NULL to shut it off. */
   CINIT(FILETIME, OBJECTPOINT, 69),
 
+  /* This points to a linked list of telnet options */
+  CINIT(TELNETOPTIONS, OBJECTPOINT, 70),
+
+  /* Max amount of cached alive connections */
+  CINIT(MAXCONNECTS, LONG, 71),
+
+  /* What policy to use when closing connections when the cache is filled
+     up */
+  CINIT(CLOSEPOLICY, LONG, 72),
+
+  /* Callback to use when CURLCLOSEPOLICY_CALLBACK is set */
+  CINIT(CLOSEFUNCTION, FUNCTIONPOINT, 73),
+
+  /* Set to explicitly use a new connection for the upcoming transfer.
+     Do not use this unless you're absolutely sure of this, as it makes the
+     operation slower and is less friendly for the network. */
+  CINIT(FRESH_CONNECT, LONG, 74),
+
+  /* Set to explicitly forbid the upcoming transfer's connection to be re-used
+     when done. Do not use this unless you're absolutely sure of this, as it
+     makes the operation slower and is less friendly for the network. */
+  CINIT(FORBID_REUSE, LONG, 75),
+
+  /* Set to a file name that contains random data for libcurl to use to
+     seed the random engine when doing SSL connects. */
+  CINIT(RANDOM_FILE, OBJECTPOINT, 76),
+
+  /* Set to the Entropy Gathering Daemon socket pathname */
+  CINIT(EGDSOCKET, OBJECTPOINT, 77),
+
+  /* Time-out connect operations after this amount of seconds, if connects
+     are OK within this time, then fine... This only aborts the connect
+     phase. [Only works on unix-style/SIGALRM operating systems] */
+  CINIT(CONNECTTIMEOUT, LONG, 78),
+
   CURLOPT_LASTENTRY /* the last unusued */
 } CURLoption;
 
@@ -431,10 +455,10 @@ typedef enum {
 
    NOTE: they return TRUE if the strings match *case insensitively*.
  */
-extern int (Curl_strequal)(const char *s1, const char *s2);
-extern int (Curl_strnequal)(const char *s1, const char *s2, size_t n);
-#define strequal(a,b) Curl_strequal(a,b)
-#define strnequal(a,b,c) Curl_strnequal(a,b,c)
+extern int (curl_strequal)(const char *s1, const char *s2);
+extern int (curl_strnequal)(const char *s1, const char *s2, size_t n);
+#define strequal(a,b) curl_strequal(a,b)
+#define strnequal(a,b,c) curl_strnequal(a,b,c)
 
 /* external form function */
 int curl_formparse(char *string,
@@ -451,9 +475,14 @@ char *curl_getenv(char *variable);
 /* Returns a static ascii string of the libcurl version. */
 char *curl_version(void);
 
+/* Escape and unescape URL encoding in strings. The functions return a new
+ * allocated string or NULL if an error occurred.  */
+char *curl_escape(char *string, int length);
+char *curl_unescape(char *string, int length);
+
 /* This is the version number */
-#define LIBCURL_VERSION "7.6.1"
-#define LIBCURL_VERSION_NUM 0x070601
+#define LIBCURL_VERSION "7.7"
+#define LIBCURL_VERSION_NUM 0x070700
 
 /* linked-list structure for the CURLOPT_QUOTE option (and other) */
 struct curl_slist {
@@ -461,184 +490,8 @@ struct curl_slist {
 	struct curl_slist	*next;
 };
 
-struct curl_slist *curl_slist_append(struct curl_slist *list, char *data);
-void curl_slist_free_all(struct curl_slist *list);
-
-/*
- * NAME	curl_init()
- *
- * DESCRIPTION
- *
- * Inits libcurl globally. This must be used before any libcurl calls can
- * be used. This may install global plug-ins or whatever. (This does not
- * do winsock inits in Windows.)
- *
- * EXAMPLE
- *
- * curl_init();
- *
- */
-CURLcode curl_init(void);
-
-/*
- * NAME	curl_init()
- *
- * DESCRIPTION
- *
- * Frees libcurl globally. This must be used after all libcurl calls have
- * been used. This may remove global plug-ins or whatever. (This does not
- * do winsock cleanups in Windows.)
- *
- * EXAMPLE
- *
- * curl_free(curl);
- *
- */
-void curl_free(void);
-
-/*
- * NAME curl_open()
- *
- * DESCRIPTION
- *
- * Opens a general curl session. It does not try to connect or do anything
- * on the network because of this call. The specified URL is only required
- * to enable curl to figure out what protocol to "activate".
- *
- * A session should be looked upon as a series of requests to a single host.  A
- * session interacts with one host only, using one single protocol.
- *
- * The URL is not required. If set to "" or NULL, it can still be set later
- * using the curl_setopt() function. If the curl_connect() function is called
- * without the URL being known, it will return error.
- *
- * EXAMPLE
- *
- * CURLcode result;
- * CURL *curl;
- * result = curl_open(&curl, "http://curl.haxx.nu/libcurl/");
- * if(result != CURL_OK) {
- *   return result;
- * }
- * */
-CURLcode curl_open(CURL **curl, char *url);
-
-/*
- * NAME curl_setopt()
- *
- * DESCRIPTION
- *
- * Sets a particular option to the specified value.
- *
- * EXAMPLE
- *
- * CURL curl;
- * curl_setopt(curl, CURL_HTTP_FOLLOW_LOCATION, TRUE);
- */
-CURLcode curl_setopt(CURL *handle, CURLoption option, ...);
-
-/*
- * NAME curl_close()
- *
- * DESCRIPTION
- *
- * Closes a session previously opened with curl_open()
- *
- * EXAMPLE
- *
- * CURL *curl;
- * CURLcode result;
- *
- * result = curl_close(curl);
- */
-CURLcode curl_close(CURL *curl); /* the opposite of curl_open() */
-
-CURLcode curl_read(CURLconnect *c_conn, char *buf, size_t buffersize,
-                   ssize_t *n);
-CURLcode curl_write(CURLconnect *c_conn, char *buf, size_t amount,
-                    size_t *n);
-
-/*
- * NAME curl_connect()
- *
- * DESCRIPTION
- *
- * Connects to the peer server and performs the initial setup. This function
- * writes a connect handle to its second argument that is a unique handle for
- * this connect. This allows multiple connects from the same handle returned
- * by curl_open().
- *
- * EXAMPLE
- *
- * CURLCode result;
- * CURL curl;
- * CURLconnect connect;
- * result = curl_connect(curl, &connect);
- */
-
-CURLcode curl_connect(CURL *curl, CURLconnect **in_connect);
-
-/*
- * NAME curl_do()
- *
- * DESCRIPTION
- *
- * (Note: May 3rd 2000: this function does not currently allow you to
- * specify a document, it will use the one set previously)
- *
- * This function asks for the particular document, file or resource that
- * resides on the server we have connected to. You may specify a full URL,
- * just an absolute path or even a relative path. That means, if you're just
- * getting one file from the remote site, you can use the same URL as input
- * for both curl_open() as well as for this function.
- *
- * In the even there is a host name, port number, user name or password parts
- * in the URL, you can use the 'flags' argument to ignore them completely, or
- * at your choice, make the function fail if you're trying to get a URL from
- * different host than you connected to with curl_connect().
- *
- * You can only get one document at a time using the same connection. When one
- * document has been received you can although request again.
- *
- * When the transfer is done, curl_done() MUST be called.
- *
- * EXAMPLE
- *
- * CURLCode result;
- * char *url;
- * CURLconnect *connect;
- * result = curl_do(connect, url, CURL_DO_NONE); */
-CURLcode curl_do(CURLconnect *in_conn);
-
-/*
- * NAME curl_done()
- *
- * DESCRIPTION
- *
- * When the transfer following a curl_do() call is done, this function should
- * get called.
- *
- * EXAMPLE
- *
- * CURLCode result;
- * char *url;
- * CURLconnect *connect;
- * result = curl_done(connect); */
-CURLcode curl_done(CURLconnect *connect);
-
-/*
- * NAME curl_disconnect()
- *
- * DESCRIPTION
- *
- * Disconnects from the peer server and performs connection cleanup.
- *
- * EXAMPLE
- *
- * CURLcode result;
- * CURLconnect *connect;
- * result = curl_disconnect(connect); */
-CURLcode curl_disconnect(CURLconnect *connect);
+struct curl_slist *curl_slist_append(struct curl_slist *, const char *);
+void curl_slist_free_all(struct curl_slist *);
 
 /*
  * NAME curl_getdate()
@@ -682,19 +535,22 @@ typedef enum {
   CURLINFO_LASTONE          = 17
 } CURLINFO;
 
-/*
- * NAME curl_getinfo()
- *
- * DESCRIPTION
- *
- * Request internal information from the curl session with this function.
- * The third argument MUST be a pointer to a long or a pointer to a char *.
- * The data pointed to will be filled in accordingly and can be relied upon
- * only if the function returns CURLE_OK.
- * This function is intended to get used *AFTER* a performed transfer, all
- * results are undefined before the transfer is completed.
- */
-CURLcode curl_getinfo(CURL *curl, CURLINFO info, ...);
+/* unfortunately, the easy.h include file needs the options and info stuff
+   before it can be included! */
+#include <curl/easy.h> /* nothing in curl is fun without the easy stuff */
+
+typedef enum {
+  CURLCLOSEPOLICY_NONE, /* first, never use this */
+
+  CURLCLOSEPOLICY_OLDEST,
+  CURLCLOSEPOLICY_LEAST_RECENTLY_USED,
+  CURLCLOSEPOLICY_LEAST_TRAFFIC,
+  CURLCLOSEPOLICY_SLOWEST,
+  CURLCLOSEPOLICY_CALLBACK,
+ 
+  CURLCLOSEPOLICY_LAST /* last, never use this */
+} curl_closepolicy;
+
 
 #ifdef  __cplusplus
 }
