@@ -5,38 +5,21 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- *  The contents of this file are subject to the Mozilla Public License
- *  Version 1.0 (the "License"); you may not use this file except in
- *  compliance with the License. You may obtain a copy of the License at
- *  http://www.mozilla.org/MPL/
+ * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
- *  Software distributed under the License is distributed on an "AS IS"
- *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
- *  License for the specific language governing rights and limitations
- *  under the License.
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
  *
- *  The Original Code is Curl.
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
  *
- *  The Initial Developer of the Original Code is Daniel Stenberg.
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
  *
- *  Portions created by the Initial Developer are Copyright (C) 1998.
- *  All Rights Reserved.
- *
- * ------------------------------------------------------------
- * Main author:
- * - Daniel Stenberg <daniel@haxx.se>
- *
- * 	http://curl.haxx.se
- *
- * $Source: /cvsroot/curl/lib/formdata.c,v $
- * $Revision: 1.11 $
- * $Date: 2000/11/17 14:06:24 $
- * $Author: bagder $
- * $State: Exp $
- * $Locker:  $
- *
- * ------------------------------------------------------------
- ****************************************************************************/
+ * $Id: formdata.c,v 1.13 2001/01/05 10:11:42 bagder Exp $
+ *****************************************************************************/
 
 /*
   Debug the form generator stand-alone by compiling this source file with:
@@ -108,16 +91,10 @@ static void GetStr(char **string,
  *
  ***************************************************************************/
 
-int curl_formparse(char *input,
-                   struct HttpPost **httppost,
-                   struct HttpPost **last_post)
-{
-  return FormParse(input, httppost, last_post);
-}
-
 #define FORM_FILE_SEPARATOR ','
 #define FORM_TYPE_SEPARATOR ';'
 
+static
 int FormParse(char *input,
 	      struct HttpPost **httppost,
 	      struct HttpPost **last_post)
@@ -315,6 +292,13 @@ int FormParse(char *input,
   return 0;
 }
 
+int curl_formparse(char *input,
+                   struct HttpPost **httppost,
+                   struct HttpPost **last_post)
+{
+  return FormParse(input, httppost, last_post);
+}
+
 static int AddFormData(struct FormData **formp,
 			void *line,
 			long length)
@@ -356,7 +340,7 @@ static int AddFormDataf(struct FormData **formp,
 }
 
 
-char *MakeFormBoundary(void)
+char *Curl_FormBoundary(void)
 {
   char *retstring;
   static int randomizer=0; /* this is just so that two boundaries within
@@ -384,7 +368,7 @@ char *MakeFormBoundary(void)
 }
 
 /* Used from http.c */ 
-void FormFree(struct FormData *form)
+void Curl_FormFree(struct FormData *form)
 {
   struct FormData *next;
   do {
@@ -417,8 +401,8 @@ void curl_formfree(struct HttpPost *form)
   } while((form=next)); /* continue */
 }
 
-struct FormData *getFormData(struct HttpPost *post,
-			     int *sizep)
+struct FormData *Curl_getFormData(struct HttpPost *post,
+                                  int *sizep)
 {
   struct FormData *form = NULL;
   struct FormData *firstform;
@@ -432,7 +416,7 @@ struct FormData *getFormData(struct HttpPost *post,
   if(!post)
     return NULL; /* no input => no output! */
 
-  boundary = MakeFormBoundary();
+  boundary = Curl_FormBoundary();
   
   /* Make the first line of the output */
   AddFormDataf(&form,
@@ -456,7 +440,7 @@ struct FormData *getFormData(struct HttpPost *post,
       /* If used, this is a link to more file names, we must then do
          the magic to include several files with the same field name */
 
-      fileboundary = MakeFormBoundary();
+      fileboundary = Curl_FormBoundary();
 
       size += AddFormDataf(&form,
 			   "\r\nContent-Type: multipart/mixed,"
@@ -552,24 +536,11 @@ struct FormData *getFormData(struct HttpPost *post,
   return firstform;
 }
 
-int FormInit(struct Form *form, struct FormData *formdata )
+int Curl_FormInit(struct Form *form, struct FormData *formdata )
 {
   if(!formdata)
     return 1; /* error */
 
-#if 0  
-  struct FormData *lastnode=formdata;
-
-  /* find the last node in the list */
-  while(lastnode->next) {
-    lastnode = lastnode->next;
-  }
-
-  /* Now, make sure that we'll send a nice terminating sequence at the end
-   * of the post. We *DONT* add this string to the size of the data since this
-   * is actually AFTER the data. */
-  AddFormDataf(&lastnode, "\r\n\r\n");
-#endif
   form->data = formdata;
   form->sent = 0;
 
@@ -577,10 +548,10 @@ int FormInit(struct Form *form, struct FormData *formdata )
 }
 
 /* fread() emulation */
-int FormReader(char *buffer,
-	       size_t size,
-	       size_t nitems,
-	       FILE *mydata)
+int Curl_FormReader(char *buffer,
+                    size_t size,
+                    size_t nitems,
+                    FILE *mydata)
 {
   struct Form *form;
   int wantedsize;
@@ -655,7 +626,7 @@ int main(int argc, char **argv)
     }
   }
 
-  form=getFormData(httppost, &size);
+  form=Curl_getFormData(httppost, &size);
 
   FormInit(&formread, form);
 
