@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: ftp.c,v 1.52 2001/02/13 13:34:16 bagder Exp $
+ * $Id: ftp.c,v 1.61 2001/03/14 18:24:07 bagder Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -77,6 +77,8 @@
 #include "krb4.h"
 #endif
 
+#include "strequal.h"
+
 #define _MPRINTF_REPLACE /* use our functions only */
 #include <curl/mprintf.h>
 
@@ -88,76 +90,8 @@
 /* easy-to-use macro: */
 #define ftpsendf Curl_ftpsendf
 
-/* returns last node in linked list */
-static struct curl_slist *slist_get_last(struct curl_slist *list)
-{
-	struct curl_slist	*item;
-
-	/* if caller passed us a NULL, return now */
-	if (!list)
-		return NULL;
-
-	/* loop through to find the last item */
-	item = list;
-	while (item->next) {
-		item = item->next;
-	}
-	return item;
-}
-
-/* append a struct to the linked list. It always retunrs the address of the
- * first record, so that you can sure this function as an initialization
- * function as well as an append function. If you find this bothersome,
- * then simply create a separate _init function and call it appropriately from
- * within the proram. */
-struct curl_slist *curl_slist_append(struct curl_slist *list, char *data)
-{
-	struct curl_slist	*last;
-	struct curl_slist	*new_item;
-
-	new_item = (struct curl_slist *) malloc(sizeof(struct curl_slist));
-	if (new_item) {
-		new_item->next = NULL;
-		new_item->data = strdup(data);
-	}
-	else {
-		fprintf(stderr, "Cannot allocate memory for QUOTE list.\n");
-		return NULL;
-	}
-
-	if (list) {
-		last = slist_get_last(list);
-		last->next = new_item;
-		return list;
-	}
-
-	/* if this is the first item, then new_item *is* the list */
-	return new_item;
-}
-
-/* be nice and clean up resources */
-void curl_slist_free_all(struct curl_slist *list)
-{
-	struct curl_slist	*next;
-	struct curl_slist	*item;
-
-	if (!list)
-		return;
-
-	item = list;
-	do {
-		next = item->next;
-		
-		if (item->data) {
-			free(item->data);
-		}
-		free(item);
-		item = next;
-	} while (next);
-}
-
-
 static CURLcode AllowServerConnect(struct UrlData *data,
+                                   struct connectdata *conn,
                                    int sock)
 {
   fd_set rdset;
@@ -187,8 +121,8 @@ static CURLcode AllowServerConnect(struct UrlData *data,
       size_t size = sizeof(struct sockaddr_in);
       struct sockaddr_in add;
 
-      getsockname(sock, (struct sockaddr *) &add, (int *)&size);
-      s=accept(sock, (struct sockaddr *) &add, (int *)&size);
+      getsockname(sock, (struct sockaddr *) &add, (socklen_t *)&size);
+      s=accept(sock, (struct sockaddr *) &add, (socklen_t *)&size);
 
       sclose(sock); /* close the first socket */
 
@@ -199,7 +133,7 @@ static CURLcode AllowServerConnect(struct UrlData *data,
       }
       infof(data, "Connection accepted from server\n");
 
-      data->secondarysocket = s;
+      conn->secondarysocket = s;
     }
     break;
   }
@@ -362,23 +296,28 @@ CURLcode Curl_ftp_connect(struct connectdata *conn)
     return CURLE_OUT_OF_MEMORY;
 
   memset(ftp, 0, sizeof(struct FTP));
-  data->proto.ftp = ftp;
+  conn->proto.ftp = ftp;
+
+  /* We always support persistant connections on ftp */
+  conn->bits.close = FALSE;
 
   /* get some initial data into the ftp struct */
   ftp->bytecountp = &conn->bytecount;
-  ftp->user = data->user;
-  ftp->passwd = data->passwd;
+
+  /* duplicate to keep them even when the data struct changes */
+  ftp->user = strdup(data->user);
+  ftp->passwd = strdup(data->passwd);
 
   if (data->bits.tunnel_thru_httpproxy) {
     /* We want "seamless" FTP operations through HTTP proxy tunnel */
-    result = Curl_ConnectHTTPProxyTunnel(conn, data->firstsocket,
-                                         data->hostname, data->remote_port);
+    result = Curl_ConnectHTTPProxyTunnel(conn, conn->firstsocket,
+                                         conn->hostname, conn->remote_port);
     if(CURLE_OK != result)
       return result;
   }
 
   /* The first thing we do is wait for the "220*" line: */
-  nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+  nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
   if(nread < 0)
     return CURLE_OPERATION_TIMEOUTED;
 
@@ -398,7 +337,7 @@ CURLcode Curl_ftp_connect(struct connectdata *conn)
        set a valid level */
     sec_request_prot(conn, data->krb4_level);
 
-    data->cmdchannel = fdopen(data->firstsocket, "w");
+    data->cmdchannel = fdopen(conn->firstsocket, "w");
 
     if(sec_login(conn) != 0)
       infof(data, "Logging in with password in cleartext!\n");
@@ -408,10 +347,10 @@ CURLcode Curl_ftp_connect(struct connectdata *conn)
 #endif
   
   /* send USER */
-  ftpsendf(data->firstsocket, conn, "USER %s", ftp->user);
+  ftpsendf(conn->firstsocket, conn, "USER %s", ftp->user);
 
   /* wait for feedback */
-  nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+  nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
   if(nread < 0)
     return CURLE_OPERATION_TIMEOUTED;
 
@@ -424,8 +363,8 @@ CURLcode Curl_ftp_connect(struct connectdata *conn)
   else if(ftpcode == 331) {
     /* 331 Password required for ...
        (the server requires to send the user's password too) */
-    ftpsendf(data->firstsocket, conn, "PASS %s", ftp->passwd);
-    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+    ftpsendf(conn->firstsocket, conn, "PASS %s", ftp->passwd);
+    nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -469,6 +408,58 @@ CURLcode Curl_ftp_connect(struct connectdata *conn)
     return CURLE_FTP_WEIRD_USER_REPLY;
   }
 
+  /* send PWD to discover our entry point */
+  ftpsendf(conn->firstsocket, conn, "PWD");
+
+  /* wait for feedback */
+  nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
+  if(nread < 0)
+    return CURLE_OPERATION_TIMEOUTED;
+
+  if(ftpcode == 257) {
+    char *dir = (char *)malloc(nread+1);
+    char *store=dir;
+    char *ptr=&buf[4]; /* start on the first letter */
+    
+    /* Reply format is like
+       257<space>"<directory-name>"<space><commentary> and the RFC959 says
+
+       The directory name can contain any character; embedded double-quotes
+       should be escaped by double-quotes (the "quote-doubling" convention).
+    */
+    if('\"' == *ptr) {
+      /* it started good */
+      ptr++;
+      while(ptr && *ptr) {
+        if('\"' == *ptr) {
+          if('\"' == ptr[1]) {
+            /* "quote-doubling" */
+            *store = ptr[1];
+            ptr++;
+          }
+          else {
+            /* end of path */
+            *store = '\0'; /* zero terminate */
+            break; /* get out of this loop */
+          }
+        }
+        else
+          *store = *ptr;
+        store++;
+        ptr++;
+      }
+      ftp->entrypath =dir; /* remember this */
+      infof(data, "Entry path is '%s'\n", ftp->entrypath);
+    }
+    else {
+      /* couldn't get the path */
+    }
+
+  }
+  else {
+    /* We couldn't read the PWD response! */
+  }
+
   return CURLE_OK;
 }
 
@@ -477,7 +468,7 @@ CURLcode Curl_ftp_connect(struct connectdata *conn)
 CURLcode Curl_ftp_done(struct connectdata *conn)
 {
   struct UrlData *data = conn->data;
-  struct FTP *ftp = data->proto.ftp;
+  struct FTP *ftp = conn->proto.ftp;
   size_t nread;
   char *buf = data->buffer; /* this is our buffer */
   struct curl_slist *qitem; /* QUOTE item */
@@ -492,7 +483,7 @@ CURLcode Curl_ftp_done(struct connectdata *conn)
   }
   else {
     if((-1 != conn->size) && (conn->size != *ftp->bytecountp) &&
-       (data->maxdownload != *ftp->bytecountp)) {
+       (conn->maxdownload != *ftp->bytecountp)) {
       failf(data, "Received only partial file");
       return CURLE_PARTIAL_FILE;
     }
@@ -502,16 +493,16 @@ CURLcode Curl_ftp_done(struct connectdata *conn)
     }
   }
 #ifdef KRB4
-  sec_fflush_fd(conn, data->secondarysocket);
+  sec_fflush_fd(conn, conn->secondarysocket);
 #endif
   /* shut down the socket to inform the server we're done */
-  sclose(data->secondarysocket);
-  data->secondarysocket = -1;
+  sclose(conn->secondarysocket);
+  conn->secondarysocket = -1;
 
   if(!data->bits.no_body) {  
     /* now let's see what the server says about the transfer we
        just performed: */
-    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -529,9 +520,9 @@ CURLcode Curl_ftp_done(struct connectdata *conn)
     while (qitem) {
       /* Send string */
       if (qitem->data) {
-        ftpsendf(data->firstsocket, conn, "%s", qitem->data);
+        ftpsendf(conn->firstsocket, conn, "%s", qitem->data);
 
-        nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+        nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
         if(nread < 0)
           return CURLE_OPERATION_TIMEOUTED;
 
@@ -560,18 +551,19 @@ CURLcode _ftp(struct connectdata *conn)
   char *buf = data->buffer; /* this is our buffer */
   /* for the ftp PORT mode */
   int portsock=-1;
-  struct sockaddr_in serv_addr;
-  char hostent_buf[8192];
 #if defined (HAVE_INET_NTOA_R)
   char ntoa_buf[64];
 #endif
 #ifdef ENABLE_IPV6
   struct addrinfo *ai;
+#else
+  struct sockaddr_in serv_addr;
+  char hostent_buf[8192];
 #endif
 
   struct curl_slist *qitem; /* QUOTE item */
   /* the ftp struct is already inited in ftp_connect() */
-  struct FTP *ftp = data->proto.ftp;
+  struct FTP *ftp = conn->proto.ftp;
 
   long *bytecountp = ftp->bytecountp;
   int ftpcode; /* for ftp status */
@@ -583,9 +575,9 @@ CURLcode _ftp(struct connectdata *conn)
     while (qitem) {
       /* Send string */
       if (qitem->data) {
-        ftpsendf(data->firstsocket, conn, "%s", qitem->data);
+        ftpsendf(conn->firstsocket, conn, "%s", qitem->data);
 
-        nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+        nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
         if(nread < 0)
           return CURLE_OPERATION_TIMEOUTED;
 
@@ -599,10 +591,27 @@ CURLcode _ftp(struct connectdata *conn)
     }
   }
 
+  if(conn->bits.reuse) {
+    /* This is a re-used connection. Since we change directory to where the
+       transfer is taking place, we must now get back to the original dir
+       where we ended up after login: */
+    ftpsendf(conn->firstsocket, conn, "CWD %s", ftp->entrypath);
+    nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
+    if(nread < 0)
+      return CURLE_OPERATION_TIMEOUTED;
+    
+    if(ftpcode != 250) {
+      failf(data, "Couldn't change back to directory %s", ftp->entrypath);
+      return CURLE_FTP_ACCESS_DENIED;
+    }
+  }
+
+
+
   /* change directory first! */
   if(ftp->dir && ftp->dir[0]) {
-    ftpsendf(data->firstsocket, conn, "CWD %s", ftp->dir);
-    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+    ftpsendf(conn->firstsocket, conn, "CWD %s", ftp->dir);
+    nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -615,9 +624,9 @@ CURLcode _ftp(struct connectdata *conn)
   if(data->bits.get_filetime && ftp->file) {
     /* we have requested to get the modified-time of the file, this is yet
        again a grey area as the MDTM is not kosher RFC959 */
-    ftpsendf(data->firstsocket, conn, "MDTM %s", ftp->file);
+    ftpsendf(conn->firstsocket, conn, "MDTM %s", ftp->file);
 
-    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -651,10 +660,10 @@ CURLcode _ftp(struct connectdata *conn)
 
     /* Some servers return different sizes for different modes, and thus we
        must set the proper type before we check the size */
-    ftpsendf(data->firstsocket, conn, "TYPE %s",
+    ftpsendf(conn->firstsocket, conn, "TYPE %s",
              (data->bits.ftp_ascii)?"A":"I");
 
-    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -665,9 +674,9 @@ CURLcode _ftp(struct connectdata *conn)
         CURLE_FTP_COULDNT_SET_BINARY;
     }
 
-    ftpsendf(data->firstsocket, conn, "SIZE %s", ftp->file);
+    ftpsendf(conn->firstsocket, conn, "SIZE %s", ftp->file);
 
-    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -709,20 +718,20 @@ CURLcode _ftp(struct connectdata *conn)
 #ifdef ENABLE_IPV6
     struct addrinfo hints, *res, *ai;
     struct sockaddr_storage ss;
-    int sslen;
+    socklen_t sslen;
     char hbuf[NI_MAXHOST];
-    char *localaddr;
+
     struct sockaddr *sa=(struct sockaddr *)&ss;
 #ifdef NI_WITHSCOPEID
     const int niflags = NI_NUMERICHOST | NI_NUMERICSERV | NI_WITHSCOPEID;
 #else
     const int niflags = NI_NUMERICHOST | NI_NUMERICSERV;
 #endif
-    unsigned char *ap;
-    unsigned char *pp;
+    char *ap;
+    char *pp;
     int alen, plen;
     char portmsgbuf[4096], tmp[4096];
-    char *p;
+
     char *mode[] = { "EPRT", "LPRT", "PORT", NULL };
     char **modep;
 
@@ -731,7 +740,7 @@ CURLcode _ftp(struct connectdata *conn)
      * I believe we should use the same address as the control connection.
      */
     sslen = sizeof(ss);
-    if (getsockname(data->firstsocket, (struct sockaddr *)&ss, &sslen) < 0)
+    if (getsockname(conn->firstsocket, (struct sockaddr *)&ss, &sslen) < 0)
       return CURLE_FTP_PORT_FAILED;
 
     if (getnameinfo((struct sockaddr *)&ss, sslen, hbuf, sizeof(hbuf), NULL, 0,
@@ -755,13 +764,13 @@ CURLcode _ftp(struct connectdata *conn)
 	continue;
 
       if (bind(portsock, ai->ai_addr, ai->ai_addrlen) < 0) {
-	close(portsock);
+	sclose(portsock);
 	portsock = -1;
 	continue;
       }
 
       if (listen(portsock, 1) < 0) {
-	close(portsock);
+	sclose(portsock);
 	portsock = -1;
 	continue;
       }
@@ -819,7 +828,7 @@ CURLcode _ftp(struct connectdata *conn)
 	  if (q)
 	    *q = '\0';
 	}
-	ftpsendf(data->firstsocket, conn, "%s |%d|%s|%s|", *modep, eprtaf,
+	ftpsendf(conn->firstsocket, conn, "%s |%d|%s|%s|", *modep, eprtaf,
 	    portmsgbuf, tmp);
       } else if (strcmp(*modep, "LPRT") == 0 || strcmp(*modep, "PORT") == 0) {
 	int i;
@@ -856,10 +865,10 @@ CURLcode _ftp(struct connectdata *conn)
 	    goto again;
 	  }
 	}
-	ftpsendf(data->firstsocket, conn, "%s %s", *modep, portmsgbuf);
+	ftpsendf(conn->firstsocket, conn, "%s %s", *modep, portmsgbuf);
       }
 
-      nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+      nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
       if (nread < 0)
 	return CURLE_OPERATION_TIMEOUTED;
 
@@ -872,7 +881,7 @@ again:;
     }
 
     if (!*modep) {
-      close(portsock);
+      sclose(portsock);
       freeaddrinfo(res);
       return CURLE_FTP_PORT_FAILED;
     }
@@ -909,7 +918,7 @@ again:;
         /* we set the secondary socket variable to this for now, it
            is only so that the cleanup function will close it in case
            we fail before the true secondary stuff is made */
-        data->secondarysocket = portsock;
+        conn->secondarysocket = portsock;
 
         memset((char *)&sa, 0, sizeof(sa));
         memcpy((char *)&sa.sin_addr,
@@ -926,7 +935,7 @@ again:;
           size = sizeof(add);
 
           if(getsockname(portsock, (struct sockaddr *) &add,
-                         (int *)&size)<0) {
+                         (socklen_t *)&size)<0) {
             failf(data, "getsockname() failed");
             return CURLE_FTP_PORT_FAILED;
           }
@@ -971,13 +980,13 @@ again:;
       sscanf( inet_ntoa(in), "%hu.%hu.%hu.%hu",
               &ip[0], &ip[1], &ip[2], &ip[3]);
 #endif
-      ftpsendf(data->firstsocket, conn, "PORT %d,%d,%d,%d,%d,%d",
+      ftpsendf(conn->firstsocket, conn, "PORT %d,%d,%d,%d,%d,%d",
             ip[0], ip[1], ip[2], ip[3],
             porttouse >> 8,
             porttouse & 255);
     }
 
-    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -998,8 +1007,8 @@ again:;
     int modeoff;
 
     for (modeoff = 0; mode[modeoff]; modeoff++) {
-      ftpsendf(data->firstsocket, conn, mode[modeoff]);
-      nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+      ftpsendf(conn->firstsocket, conn, mode[modeoff]);
+      nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
       if(nread < 0)
 	return CURLE_OPERATION_TIMEOUTED;
 
@@ -1021,9 +1030,10 @@ again:;
       struct addrinfo *res;
 #else
       struct hostent *he;
-#endif
-      char *str=buf,*ip_addr;
       char *hostdataptr=NULL;
+      char *ip_addr;
+#endif
+      char *str=buf;
 
       /*
        * New 227-parser June 3rd 1999.
@@ -1057,12 +1067,12 @@ again:;
          * previous lookup.
          */
 #ifdef ENABLE_IPV6
-        res = conn->res;
+        res = conn->hp;
 #else
         he = conn->hp;
 #endif
         connectport =
-          (unsigned short)data->port; /* we connect to the proxy's port */
+          (unsigned short)conn->port; /* we connect to the proxy's port */
       }
       else {
         /* normal, direct, ftp connection */
@@ -1081,15 +1091,15 @@ again:;
       }
 	
 #ifdef ENABLE_IPV6
-      data->secondarysocket = -1;
+      conn->secondarysocket = -1;
       for (ai = res; ai; ai = ai->ai_next) {
 	/* XXX for now, we can do IPv4 only */
 	if (ai->ai_family != AF_INET)
 	  continue;
 
-	data->secondarysocket = socket(ai->ai_family, ai->ai_socktype,
+	conn->secondarysocket = socket(ai->ai_family, ai->ai_socktype,
 	    ai->ai_protocol);
-	if (data->secondarysocket < 0)
+	if (conn->secondarysocket < 0)
 	  continue;
 
 	if(data->bits.verbose) {
@@ -1114,21 +1124,21 @@ again:;
 	  }
 	}
 
-	if (connect(data->secondarysocket, ai->ai_addr, ai->ai_addrlen) < 0) {
-	  close(data->secondarysocket);
-	  data->secondarysocket = -1;
+	if (connect(conn->secondarysocket, ai->ai_addr, ai->ai_addrlen) < 0) {
+	  close(conn->secondarysocket);
+	  conn->secondarysocket = -1;
 	  continue;
 	}
 
 	break;
       }
 
-      if (data->secondarysocket < 0) {
+      if (conn->secondarysocket < 0) {
 	failf(data, strerror(errno));
         return CURLE_FTP_CANT_RECONNECT;
       }
 #else
-      data->secondarysocket = socket(AF_INET, SOCK_STREAM, 0);
+      conn->secondarysocket = socket(AF_INET, SOCK_STREAM, 0);
 
       memset((char *) &serv_addr, '\0', sizeof(serv_addr));
       memcpy((char *)&(serv_addr.sin_addr), he->h_addr, he->h_length);
@@ -1202,7 +1212,7 @@ again:;
       if(hostdataptr)
         free(hostdataptr);
 
-      if (connect(data->secondarysocket, (struct sockaddr *) &serv_addr,
+      if (connect(conn->secondarysocket, (struct sockaddr *) &serv_addr,
                   sizeof(serv_addr)) < 0) {
         switch(errno) {
 #ifdef ECONNREFUSED
@@ -1213,7 +1223,7 @@ again:;
 #endif
 #ifdef EINTR
         case EINTR:
-          failf(data, "Connection timeouted to ftp server");
+          failf(data, "Connection timed out to ftp server");
           break;
 #endif
         default:
@@ -1226,7 +1236,7 @@ again:;
 
       if (data->bits.tunnel_thru_httpproxy) {
         /* We want "seamless" FTP operations through HTTP proxy tunnel */
-        result = Curl_ConnectHTTPProxyTunnel(conn, data->secondarysocket,
+        result = Curl_ConnectHTTPProxyTunnel(conn, conn->secondarysocket,
                                              newhost, newport);
         if(CURLE_OK != result)
           return result;
@@ -1241,10 +1251,10 @@ again:;
   if(data->bits.upload) {
 
     /* Set type to binary (unless specified ASCII) */
-    ftpsendf(data->firstsocket, conn, "TYPE %s",
+    ftpsendf(conn->firstsocket, conn, "TYPE %s",
           (data->bits.ftp_ascii)?"A":"I");
 
-    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -1273,9 +1283,9 @@ again:;
         /* we could've got a specified offset from the command line,
            but now we know we didn't */
 
-        ftpsendf(data->firstsocket, conn, "SIZE %s", ftp->file);
+        ftpsendf(conn->firstsocket, conn, "SIZE %s", ftp->file);
 
-        nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+        nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
         if(nread < 0)
           return CURLE_OPERATION_TIMEOUTED;
 
@@ -1332,11 +1342,11 @@ again:;
     /* Send everything on data->in to the socket */
     if(data->bits.ftp_append)
       /* we append onto the file instead of rewriting it */
-      ftpsendf(data->firstsocket, conn, "APPE %s", ftp->file);
+      ftpsendf(conn->firstsocket, conn, "APPE %s", ftp->file);
     else
-      ftpsendf(data->firstsocket, conn, "STOR %s", ftp->file);
+      ftpsendf(conn->firstsocket, conn, "STOR %s", ftp->file);
 
-    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -1347,7 +1357,7 @@ again:;
     }
 
     if(data->bits.ftp_use_port) {
-      result = AllowServerConnect(data, portsock);
+      result = AllowServerConnect(data, conn, portsock);
       if( result )
         return result;
     }
@@ -1360,7 +1370,7 @@ again:;
     Curl_pgrsSetUploadSize(data, data->infilesize);
 
     result = Curl_Transfer(conn, -1, -1, FALSE, NULL, /* no download */
-                      data->secondarysocket, bytecountp);
+                      conn->secondarysocket, bytecountp);
     if(result)
       return result;
       
@@ -1392,16 +1402,16 @@ again:;
       else if(from < 0) {
         /* -Y */
         totalsize = -from;
-        data->maxdownload = -from;
+        conn->maxdownload = -from;
         data->resume_from = from;
         infof(data, "FTP RANGE the last %d bytes\n", totalsize);
       }
       else {
         /* X-Y */
         totalsize = to-from;
-        data->maxdownload = totalsize+1; /* include the last mentioned byte */
+        conn->maxdownload = totalsize+1; /* include the last mentioned byte */
         data->resume_from = from;
-        infof(data, "FTP RANGE from %d getting %d bytes\n", from, data->maxdownload);
+        infof(data, "FTP RANGE from %d getting %d bytes\n", from, conn->maxdownload);
       }
       infof(data, "range-download from %d to %d, totally %d bytes\n",
             from, to, totalsize);
@@ -1414,9 +1424,9 @@ again:;
       dirlist = TRUE;
 
       /* Set type to ASCII */
-      ftpsendf(data->firstsocket, conn, "TYPE A");
+      ftpsendf(conn->firstsocket, conn, "TYPE A");
 	
-      nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+      nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
       if(nread < 0)
         return CURLE_OPERATION_TIMEOUTED;
 	
@@ -1429,16 +1439,16 @@ again:;
          better used since the LIST command output is not specified or
          standard in any way */
 
-      ftpsendf(data->firstsocket, conn, "%s",
+      ftpsendf(conn->firstsocket, conn, "%s",
             data->customrequest?data->customrequest:
             (data->bits.ftp_list_only?"NLST":"LIST"));
     }
     else {
       /* Set type to binary (unless specified ASCII) */
-      ftpsendf(data->firstsocket, conn, "TYPE %s",
+      ftpsendf(conn->firstsocket, conn, "TYPE %s",
                (data->bits.ftp_ascii)?"A":"I");
 
-      nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+      nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
       if(nread < 0)
         return CURLE_OPERATION_TIMEOUTED;
 
@@ -1457,9 +1467,9 @@ again:;
          * of the file we're gonna get. If we can get the size, this is by far
          * the best way to know if we're trying to resume beyond the EOF.  */
 
-        ftpsendf(data->firstsocket, conn, "SIZE %s", ftp->file);
+        ftpsendf(conn->firstsocket, conn, "SIZE %s", ftp->file);
 
-        nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+        nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
         if(nread < 0)
           return CURLE_OPERATION_TIMEOUTED;
 
@@ -1501,9 +1511,9 @@ again:;
         infof(data, "Instructs server to resume from offset %d\n",
               data->resume_from);
 
-        ftpsendf(data->firstsocket, conn, "REST %d", data->resume_from);
+        ftpsendf(conn->firstsocket, conn, "REST %d", data->resume_from);
 
-        nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+        nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
         if(nread < 0)
           return CURLE_OPERATION_TIMEOUTED;
 
@@ -1513,10 +1523,10 @@ again:;
         }
       }
 
-      ftpsendf(data->firstsocket, conn, "RETR %s", ftp->file);
+      ftpsendf(conn->firstsocket, conn, "RETR %s", ftp->file);
     }
 
-    nread = Curl_GetFTPResponse(data->firstsocket, buf, conn, &ftpcode);
+    nread = Curl_GetFTPResponse(conn->firstsocket, buf, conn, &ftpcode);
     if(nread < 0)
       return CURLE_OPERATION_TIMEOUTED;
 
@@ -1580,7 +1590,7 @@ again:;
         size = downloadsize;
 
       if(data->bits.ftp_use_port) {
-        result = AllowServerConnect(data, portsock);
+        result = AllowServerConnect(data, conn, portsock);
         if( result )
           return result;
       }
@@ -1588,7 +1598,7 @@ again:;
       infof(data, "Getting file with size: %d\n", size);
 
       /* FTP download: */
-      result=Curl_Transfer(conn, data->secondarysocket, size, FALSE,
+      result=Curl_Transfer(conn, conn->secondarysocket, size, FALSE,
                       bytecountp,
                       -1, NULL); /* no upload here */
       if(result)
@@ -1617,7 +1627,7 @@ CURLcode Curl_ftp(struct connectdata *conn)
   int dirlength=0; /* 0 forces strlen() */
 
   /* the ftp struct is already inited in ftp_connect() */
-  ftp = data->proto.ftp;
+  ftp = conn->proto.ftp;
 
   /* We split the path into dir and file parts *before* we URLdecode
      it */
@@ -1706,3 +1716,16 @@ size_t Curl_ftpsendf(int fd, struct connectdata *conn, char *fmt, ...)
 }
 
 
+CURLcode Curl_ftp_disconnect(struct connectdata *conn)
+{
+  struct FTP *ftp= conn->proto.ftp;
+
+  if(ftp->user)
+    free(ftp->user);
+  if(ftp->passwd)
+    free(ftp->passwd);
+  if(ftp->entrypath)
+    free(ftp->entrypath);
+
+  return CURLE_OK;
+}
